import {
  csv,
  select,
  json,
  geoPath,
  geoAlbersUsa,
  zoom,
  event,
  scaleOrdinal,
} from 'd3';


import { feature } from 'topojson';
import { colorLegend } from './colorLegend'
import { chart } from './chart'

const colorScale = scaleOrdinal()
  .domain(["McDonald's", "Burger King", "Taco Bell", "Wendy's", "Arby's", "KFC","Subway","other"])
  .range(['#fffa05','#b21dc3', '#2494fd','#fe2323','#fcc725','#fb7217','#7bfd24','#545454']);

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');


const projection = geoAlbersUsa().scale(900).translate([width/2+50, height/2+15]);;
const pathGenerator = geoPath().projection(projection);


const mapg = svg.append('g');
const clg = svg.append('g');
const stats = svg.append('g');
const chartBar = svg.append('g'); 
const chartYlabel = svg.append('g');
const chartTitle = svg.append('g'); 


let clickObj_cl = null;
let off_chart = 0;
let centered;

 
svg.call(zoom()
  .on('zoom', ()=>{mapg.attr('transform', event.transform)}));
  
/*************make title********************/

const make_title = (input) => {
   const title_pos = 50
    mapg.append('text')
     .attr("class", 'mytitle')
     .attr('font-size', title_pos*1)
     .attr('x', width/2+40)
     .attr('y', title_pos)
     .text(input)
};


/*************make dot******************/

const make_dot = (data) => {
    const realpos = data.map( d => 
          [+d.longitude, +d.latitude, d.name]);
    mapg.selectAll("circle").data(realpos)
       .enter().append("circle")
         .attr("cx", d => projection(d)[0])
         .attr("cy", d => projection(d)[1])
         .attr("fill", d => colorScale(d[2]))
    	 .merge(mapg.selectAll("circle"))
         .attr('class','dots')
         .attr('stroke','black')
         .attr('stroke-width',d => d[2] === clickObj_cl ? 0.4 : 0)
         .attr("r", d => d[2] === clickObj_cl ? 3 : 2)
         .attr("opacity", d => (!clickObj_cl || d[2] === clickObj_cl) ? 0.8 : 0)
};

/*************plotChart****************/

const plotChart = (data, target) =>{
    data = data.filter(d => d.province ==target)
    let statis = {}
    data.forEach(d => statis[d.name] = 0)
    data.forEach(d=> statis[d.name] = statis[d.name] +1)
    let res = Object.keys(statis).map( d => ([ d, +statis[d]]) )
    res = res.filter(d => d[1]>0)
    res = res.sort( (a, b) => {
    let a1= a[1], b1= b[1]
    if(a1== b1) return 0;
    return a1> b1? 1: -1;
    }).reverse()

    let count = 0
    let res_plot = []
    while (count < Math.min(5,res.length)) {
    res_plot.push(res[count])
    count = count + 1
    } 
   
    chart(
      res_plot,{
      selection: chartBar,
      selection2:chartYlabel,
      selection3:chartTitle,
      target,
      off_chart})
  
};
/************************************/

Promise.all([ 
    json('https://gist.githubusercontent.com/mbostock/4090846/raw/d534aba169207548a8a3d670c9c2cc719ff05c47/us.json'), csv('https://vizhub.com/NTCHANG1289/datasets/fastfoodrestaurants_fix_1028.csv')])
  .then(([json, data]) => {
        let data1 = data.filter(d => 
        d.longitude < -50 & d.latitude > 23 |
        d.province =="HI")
        
        let data2 = JSON.parse(JSON.stringify(data1));
        
        data1.forEach(d => 
        {if (colorScale.domain().includes(d.name)==0){ 
          d.name="other"}
        })
         
      /**********on click color legend*******************/

      const onClick_cl = d => { 
           clickObj_cl = d
           make_colorlegend()
           make_dot(data1)
      };  
      /**********make color legend******************/

      const make_colorlegend= () => {
        const clfac = 10
        clg
          .attr('transform',`translate(${width-150}, ${height-200})`)
          .call(colorLegend,{
             colorScale,
             circleRadius:clfac,
             spacing:clfac*2+5,
             textsize: clfac*1.8,
             textOffset:clfac*2,
             onClick_cl,
             clickObj_cl
          });
      };
      
      /*************make bottom********************/

      const make_bottom = () => {
         const bottom = svg.append('g')
           .attr('class','bottom')
           .on('click', d=>{
             off_chart=1,plotChart(data2,'CA')})
         bottom.append('text')
           .text('close chart')   
           .attr('font-size', 18)
           .attr('fill', 'white')
           .attr('opacity',1)
           .attr('x', 20)
           .attr('y', 80)
      };



  
      /**********on click map****************/

      const onClick_map = d => { 
      off_chart = 0   
      plotChart(data2,d.properties.province) 
        
      let x, y, k

      if (d && centered !== d) {
        var centroid = pathGenerator.centroid(d);
        x = centroid[0];
        y = centroid[1];
        k =3;
        centered = d;
      } else {
        x = width / 2;
        y = height / 2;
        k = 1;
        centered = null;
      }
      
      mapg.selectAll("path").classed("active", 
        centered && function(d) { return d === centered;});

      mapg.transition()
          .duration(750)
          .attr("transform", "translate(" + width / 2 + "," + height / 2 + ")scale(" + k + ")translate(" + -x + "," + -y + ")")
          }
  
       
       
  
      /*************make map****************/
 
      const make_map = (json,data) => {

          const rowByID = data.reduce((accumulator, d) => {
            accumulator[d.stateid] = d;
            return accumulator;
          }, {});

          const statespv = feature(json, json.objects.states);

          statespv.features.forEach(d => {
            Object.assign(d.properties, rowByID[d.id]);
            });

          mapg.selectAll('path').data(statespv.features)
            .enter().append('path')
              .attr('class', 'states')
              .attr('d', pathGenerator)
              .on('click', d => onClick_map(d))
        			.on("mouseover", (d,i)=> {
            		select(this)
                .attr("fill",'red')})
            .append('title')
        			
              .text(d => d.properties.province)
    				 

      };
      /*************************************/

    

make_title('American fastfood map');
make_map(json,data);
make_colorlegend();
make_dot(data1);
make_bottom();

});
